# Signal Detection Example 1 - closed form calculation

load("GabResp.Rdata")  # load data file
str(GabResp)

table(GabResp)   # cross-classify counts of each level 

# Restructure into data frame  
Gb.df <- data.frame( Resp = GabResp %in% c("H", "FA"),	
                  Stim = GabResp %in% c("H", "M"))
str(Gb.df)

head(Gb.df) # TRUE = Present (1) ; FALSE = ABSENT (0)
   
( Gb.tab <- with(Gb.df, table(Stim, Resp)) ) # tabulate 4 cases

prop.table(Gb.tab, 1)  # normalize on row sums

diff(qnorm(prop.table(Gb.tab, 1)[, 2]))

# Signal Detection Example 2 - ML solution

X <- model.matrix(~ Stim, data = Gb.df)  # create model matrix

llik <- function(b, X, Y){
	p <- pnorm(X %*% b)
	-sum(Y * log(p) + (1 - Y) * log(1 - p))
}

optim(c(0, 1), llik, X = X, Y = Gb.df$Resp)$par
optim(c(0, 1), llik, X = X, Y = Gb.df$Resp, control = list(reltol = 1e-12))$par

Gb.opt <- optim(c(0, 1), llik, X = X, Y = Gb.df$Resp, control = list(reltol = 1e-12),
	hessian = TRUE)

Gb.opt$par  # estimated parameters
sqrt(diag(solve(Gb.opt$hessian)))  #  approximate standard errors

# Signal Detection Example 3 - GLM solution
Gb.glm <- glm(Resp ~ Stim, binomial(probit), Gb.df)
summary(Gb.glm)

# Example 4: More complex experimental design: Anova
Hypno.df <- read.table(textConnection("
  Stim Number Resp       Cond
1  Old     69  Yes     Normal
2  New     31  Yes     Normal
3  Old     31   No     Normal
4  New     69   No     Normal
5  Old     89  Yes Hypnotized
6  New     59  Yes Hypnotized
7  Old     11   No Hypnotized
8  New     41   No Hypnotized"), header = TRUE)
Hypno.df$Cond <- relevel(Hypno.df$Cond, "Normal")
str(Hypno.df)

m2 <- glm(Resp ~ Stim * Cond, binomial(probit), Hypno.df, 
		weights = Number)
m1 <- update(m2, . ~ . - Stim:Cond)
m0 <- update(m1, . ~ . - Cond)
anova(m0, m1, m2, test = "Chisq")

library(effects)
allEffects(m2)
summary(m2)
plot(allEffects(m2), type = "link")

# Complete Separation Example 4
oops <- options(warn = 0)
CS.df <- data.frame(Present = c(10000, 100),
					Absent = c(0, 9900),
					Stim = factor(c(1, 0)))
summary(glm(cbind(Present, Absent) ~ Stim, binomial, CS.df))

CS1.df <- data.frame(Present = c(10000, 100),
					Absent = c(1, 9900),
					Stim = factor(c(1, 0)))
summary(glm(cbind(Present, Absent) ~ Stim, binomial, CS1.df))


#  Rating Scale Example 5 - estimated points
#data(Faces, package = "MPDiR")
load("Faces.Rdata")
Faces.tab <- xtabs(~ sibs + SimRating, Faces)  # tabulate sib status w/ ratings
Faces.csum <- apply(Faces.tab[, 11:1], 1, cumsum)[11:1, ]
Faces.ROC <- Faces.csum/Faces.csum[1, ]

par(pty = "s")
plot(Faces.ROC, xlim = c(0, 1), ylim = c(0, 1), 
	xlab = expression(P[FA]), ylab = expression(P[H]), main = "ROC curve")

# Fitting cumulative link model
library(ordinal)
m1 <- clm(ordered(SimRating) ~ sibs, data = Faces, link = "probit")
summary(m1)
m2 <- clm(ordered(SimRating) ~ sibs, scale = ~ sibs, data = Faces, link = "probit")
summary(m2)
anova(m1, m2)

Faces.zROC <- qnorm(Faces.ROC)
Faces.dp <- apply(Faces.zROC[-1, ], 1, diff)
c(dp = mean(Faces.dp), SDdp =sd(Faces.dp))
dp.est <- coef(lm(Faces.zROC[-1, 2] ~ offset(Faces.zROC[-1, 1])))

par(mfrow = c(1, 2), pty = "s")
plot(Faces.zROC[-1, ], xlim = c(-2, 2), ylim = c(-2, 2), 
	xlab = expression(Phi^{-1}[FA]), ylab = expression(Phi^{-1}[H]), 
	main = "zROC curve")
abline(dp.est, 1)
abline(0, 1, col = "grey")
abline(v = 0, h = 0, col = "grey", lty = 2)
arrows(0, dp.est, -2, dp.est, col = "red", lwd = 2, length = 0.1)
text(-2, 1.4, "d'", adj = 0)
plot(Faces.ROC, xlim = c(0, 1), ylim = c(0, 1), 
	xlab = expression(P[FA]), ylab = expression(P[H]), main = "ROC curve")
xx <- seq(-3, 3, len = 100)
lines(pnorm(xx, low = FALSE), pnorm(xx, mean = dp.est, low = FALSE))
abline(0, 1, col = "grey")

library(ordinal)
m1 <- clm(ordered(SimRating) ~ sibs, data = Faces, link = "probit")
summary(m1)
m2 <- clm(ordered(SimRating) ~ sibs, scale = ~sibs, data = Faces, link = "probit")
summary(m2)
anova(m1, m2)

# GLMM example 6
load("Faces2.Rdata")
str(Faces2)

summary(glm(Resp ~ Stim, binomial(probit), Faces2))

#library(lme4)  # current CRAN version
library(lme4, lib.loc = "~/projets/devlib2") # devel version 1.0-0 w/ bells & whistles
       
# crossed random effects of Obs and Image
gm1 <- glmer(Resp ~ Stim + (1 | Obs) + (1 | Image), Faces2, binomial(probit))
summary(gm1)

# centering explanatory variable
Faces2$LStim <- with(Faces2, ifelse(Stim == "P", 0.5, -0.5))
gm1c <- glmer(Resp ~ LStim + (1 | Obs) + (1 | Image), Faces2, binomial(probit))
summary(gm1c) 

gm2a <- glmer(Resp ~ LStim + (LStim + 0 | Obs) + 
	(1 | Obs) + (1 | Image), Faces2, binomial(probit))
gm2b <- glmer(Resp ~ LStim + (1 | Obs) + (1 | Image) + 
	(LStim + 0 | Image), Faces2, binomial(probit))
gm3 <- glmer(Resp ~ LStim + (LStim + 0 | Obs) + (LStim + 0| Image) +
	(1 | Obs) + (1 | Image), 
	Faces2, binomial(probit))
gm4 <- glmer(Resp ~ LStim + (LStim | Obs) + (LStim | Image), 
	Faces2, binomial(probit))  # with correlations among rx's

anova(gm1c, gm2a, gm3, gm4)
anova(gm1c, gm2b, gm3, gm4)

# caterpillar plots of random effects
dotplot(ranef(gm4, condVar = TRUE, whichel = "Obs"), )
dev.new()
dotplot(ranef(gm4, condVar = TRUE, whichel = "Image"))


#  example of profile plots
gm1c.pr <- profile(gm1c)
gm1.pr <- profile(gm1)
xyplot(gm1c.pr)
splom(gm1.pr)
splom(gm1c.pr)
densityplot(gm1c.pr)
   
# MLDS - example 7
library(MLDS)
kk.mlds <- mlds(kk1)
plot(kk.mlds, type = "b", xlab = expression(r^2), ylab = "Difference Scale")
plot(kk.mlds, standard.scale = TRUE,
	xlab = expression(r^2), ylab = "Difference Scale")
xx <- seq(0, 1, len = 100)
lines(xx, xx^2, lwd = 2, col = "blue")

mlds(~ x^p, p = c(2, 0.1), data = kk1)$par


